<?php

class EveryBoard_Article_Helper {

    // Array that contains all "cached" article objects, key is the uuid.
    protected static $board_article_array = [];

    protected static $oc_list_uuid_structure = [];

    /**
     * Iterates over a board-json and makes ONE request to Open Content to fetch all the manual articles for a board.
     * @param $board_json
     */
    public static function prefetch_board_articles( $board_json ) {

        if ( $board_json !== null && array_key_exists( 'rows', $board_json ) ) {

            $oc_api = new OcAPI();
            $oc_list_structure = [];

            $article_uuids = [];

            foreach ($board_json['rows'] as $row) {

                $cols = isset($row['cols']) ? $row['cols'] : [];

                foreach ($cols as $col) {

                    $content = isset($col['content']) ? $col['content'] : [];

                    foreach ($content as $content_item) {

                        // Continue if content has no type
                        if ( ! isset($content_item['type'])) {
                            continue;
                        }

                        // Set uuid and move on if type article
                        if ($content_item['type'] === 'oc_article') {
                            $article_uuids[] = $content_item['oc_uuid'];
                            continue;
                        }

                        // Construct data with the OC Lists and positions that are being used in the board.
                        if ($content_item['type'] === 'oclist_widget') {

                            $list_uuid = isset($content_item['settings']['list']) ? $content_item['settings']['list'] : '0';

                            if ($list_uuid === '0' || ! isset($content_item['settings']['listposition'], $content_item['settings']['listlimit'])) {
                                continue;
                            }

                            // Preset list from list uuid if it don't already exists
                            if ( ! isset($oc_list_structure[$list_uuid]) ) {
                                $oc_list_structure[$list_uuid] = [];
                            }

                            $list_start = (int)$content_item['settings']['listposition'] - 1;
                            $list_start = $list_start < 0 ? 0 : $list_start;

                            $list_limit = (int)$content_item['settings']['listlimit'];
                            $list_stop = $list_start + $list_limit;

                            // Iterate through selected positions and add article_uuids
                            for ($i = $list_start; $i < $list_stop; $i++) {

                                if ( ! in_array($i, $oc_list_structure[$list_uuid], true) ) {
                                    $oc_list_structure[$list_uuid][] = $i;
                                }
                            }
                        }
                    }
                }
            }

            // Fetch article uuids from OC List to be able to include these articles in the batch query.
            foreach( $oc_list_structure as $list_id => $positions ) {

                $articles = OcList::get_approved_list_articles($list_id);

                $oc_list_articles = [];
                foreach( $positions as $position ) {
                    $article_uuid = isset( $articles[$position], $articles[$position]->uuid ) ? $articles[$position]->uuid[0] : null;
                    $oc_list_articles[$position] = $article_uuid;

                    // Add uuid to query
                    if( $article_uuid !== null ) {
                        $article_uuids[] = $article_uuid;
                    }
                }

                self::$oc_list_uuid_structure[$list_id] = $oc_list_articles;
            }

            $number_of_articles = \count($article_uuids);

            // Only prefetch articles if any found in board.
            if ( $number_of_articles > 0 ) {
                $article_query = 'uuid:' . implode(' OR uuid:', $article_uuids);
                $oc_api->prepare_query( $article_query, 0, $number_of_articles );
                $prefetched_articles = $oc_api->get_oc_articles();

                foreach ( $prefetched_articles as $prefetched_article ) {
                    $article_uuid = $prefetched_article->uuid[0];
                    self::$board_article_array[$article_uuid] = $prefetched_article;
                }
            }
        }

    }

    /**
     * Function that prefetches articles from OC by an array of uuids.
     * If it's able to fetch it returns true, otherwise null
     *
     * @param $uuids
     *
     * @return bool|null
     */
    public static function prefetch_board_list_articles( $uuids ) {
        $oc_api = new OcAPI();

        if( empty( $uuids ) ) {
            return null;
        }

        $query = 'uuid:' . implode(' OR uuid:', $uuids);
        $oc_api->prepare_query( "({$query})", 0, count($uuids) );
        $prefetched_articles = $oc_api->get_oc_articles();

        if ( !empty( $prefetched_articles ) ) {
            foreach ( $prefetched_articles as $prefetched_article ) {
                $article_uuid = $prefetched_article->uuid[0];
                self::$board_article_array[$article_uuid] = $prefetched_article;
            }

            return true;
        }

        return null;
    }

    /**
     * If the article is in the array it is returned straight from it, otherwise fetch from OC and add to array.
     *
     * @param string $uuid
     * @param bool   $force_fetch
     *
     * @return OcArticle|null
     */
    public static function get_oc_article($uuid, $force_fetch = true) {

        // Return if found
        if (isset(self::$board_article_array[$uuid])) {
            return self::$board_article_array[$uuid];
        }

        if ( ! $force_fetch) {
            return null;
        }

        $article = self::force_fetch_oc_article($uuid);

        // If forced store the resulted article for later use
        if ($article instanceof OcArticle) {
            self::$board_article_array[$uuid] = $article;

            return $article;
        }

        return null;

    }

    /**
     * Helps out by placing prefetched articles in correct order and place from a oc list structure.
     *
     * @param $list
     * @param $listposition
     * @param $limit
     *
     * @return array
     */
    public static function get_prefetched_oclist_articles($list, $listposition, $limit = 1) {

        if ( ! isset(self::$oc_list_uuid_structure[$list])) {
            return [];
        }

        $articles = [];
        for ($i = $listposition; $i < $listposition + $limit; $i++) {

            if ( ! isset(self::$oc_list_uuid_structure[$list][$i])) {
                $articles[] = null;
                continue;
            }

            $uuid = self::$oc_list_uuid_structure[$list][$i];

            $articles[] = isset( self::$board_article_array[$uuid] ) ? self::$board_article_array[$uuid] : null;
        }

        return $articles;
    }

    /**
     * Trying to fetch the article from OC/Cache instead of returning it from the prefetched array
     *
     * @param $uuid
     *
     * @return OcArticle|null
     */
    public static function force_fetch_oc_article( $uuid ) {
        $oc_api = new OcAPI();
        $article = $oc_api->get_single_article( $uuid );

        return isset( $article['article'] ) ? $article['article'] : null;
    }

    /**
     * @param $uuid
     *
     * @return OcArticle|null
     */
    public static function force_fetch_search_oc_article($uuid) {
        $oc_api = new OcAPI();
        $article = $oc_api->search(['q' => "uuid:{$uuid}"], false, true);

        return isset($article[0]) ? $article[0] : null;
    }
}